function [ background, low_vent, mid_vent, high_vent ] = FindMIPThresholdValuesKmeans(f19_lung)
%Determines low, medium, high ventilation threshold using MIP
% uses 1st and last slice of MIP which do not have f19 signal
% f19_lung = MIP image with outside lung set to value of 0

[r,c,v] = ind2sub(size(f19_lung),find(f19_lung > 0));
lung_data=f19_lung(f19_lung > 0);


%% Compute k-means clustering

idx = kmeans (lung_data,8,'MaxIter',300);
%Sort k-means to correct indecies (kmeans assigns random indecies)
[new_idx]=Sort1DKmean(lung_data,idx);
kdata=[lung_data,new_idx];

background = max(kdata(kdata(:,2)==1))+.0001; % group 1
low_vent  = max(kdata(kdata(:,2)==2))+.0001; % group 2
mid_vent  = max(kdata(kdata(:,2)==5))+.0001; % group 3 and 4 and 5
high_vent = max(kdata(kdata(:,2)==8))+.0001; % group 6 and 7 and 8
end