%% Initialize
clear;clc;
home = pwd;


%% Choose Parameters for Running
% Choose patients
patients = [ '005'; '006'; '007';'008'; '009'; '010'; '011'; '012';'013';'015']; %;'013B';'018A';'028A'];
PlotVDPBool = 0;
PlotMIPBool = 0;
PlotMIPImageBool = 0; %%
SaveMIPImageBool = 0;
% RGB Image
PlotRGBImageBool = 0; %%
SaveRGBImageBool = 0;
% Six Segment Image
PlotSixSegmentModelBool = 0;
SaveSixSegmentModelBool =0;
% F19 histogram image
PlotF19HistogramBool = 0;
% Save CSV ventilation data to file
WriteCSVVentilationDataBool = 0;
% Save tau1 data to file
WriteTau1DataBool = 0;
LoadTau1DataBool = 0;
% B1correction
B1correctBoolF = 0;
B1correctBoolXe = 0;
LoadB1correctBoolF = 0;
LoadB1correctBoolXe = 0;
LoadB1correctHealthyF = 0;
LoadB1correctPhantom = 0;
XeCalcBool = 1;
SaveBiasFieldBool = 0;
MIPThresholdNormalBool = 1;
MIPThresholdNoiseBool = 0;
XenonThresholdNormalBool = 1;
XenonKmeansBool = 0;
MIPKmeansBool = 0;
KmeansTauBool = 0;
SaveTau1DataBool = 0;
SaveTau2DataBool = 0;
SaveTauVentMapBool = 0;
SaveVDPandThresh = 1;
SaveVentBool= 0;
SaveUnventilatedMap = 0;
addpath('./functions')
addpath('./MICO_v0/MICO_v0/MICO_3D')


%% Loop through selected subjects
for i = 1:size(patients,1)
    if XeCalcBool
        %% Load and Format Initial Imaging Data
        % load 129Xe ventilaion
        cd('./data')
        filename = strcat('2569-',patients(i,:),'_XE.mat');
        load(filename);
        fixed = double(FUNC);
        
        
        % load anatomical 1h ROI mri
        filename = strcat('2569-',patients(i,:),'_ROI.mat');
        load(filename);
        moving = double(imresize(ANATOM_ROI,[size(FUNC,1),size(FUNC,2)]));
        
        % load anatomical 1h mri
        filename = strcat('2569-',patients(i,:),'_XEanat.mat');
        load(filename);
        if exist('ANATOM')
            ANAT = ANATOM;
        end
        
        % back to home directory and add functions path
        cd(home)
        
        %% Bias Correction
        if B1correctBoolXe
            [fixed,b] = MICO_3D_lung(fixed,moving);
            if SaveBiasFieldBool
                VarDirectory = strcat('./outputs/BiasField_Xe129/');  mkdir(VarDirectory);
                VarName = strcat('BiasField_Patient_',string(patients(i,:)));
                FileName = char(strcat(VarDirectory,VarName,'.mat'));
                save(FileName,'b','fixed')
            end
        end
        if LoadB1correctBoolXe
            VarDirectory = strcat('./outputs/BiasField_Xe129/');  mkdir(VarDirectory);
            VarName = strcat('BiasField_Patient_',string(patients(i,:)));
            FileName = char(strcat(VarDirectory,VarName,'.mat'));
            load(FileName);
        end
        Xe_lung = fixed.*moving;
        %% Plot 129Xe Histogram on Figure 4 if selected
        if PlotF19HistogramBool
            [P90data(i) , meantop10data(i)] = PlotF19Histogram(patients(i,:), Xe_lung);
        end
        
        %% Compute Values for lowvent, midvent, highvent
        if XenonThresholdNormalBool
            [background_Xe(i), low_vent_Xe(i), mid_vent_Xe(i), high_vent_Xe(i), max_vent_Xe(i)] = FindMIPThresholdValuesXe(fixed , Xe_lung);
        end
        
        backgrounda = reshape(fixed(:,:,1),1,[]);
        bgd_mean(i,2) = mean(backgrounda);
        bgd_std(i,2)  =  std(backgrounda);
        %% Compute maps for lowvent, midvent, highvent using kmeans
        if XenonKmeansBool
            [xenon_vent_map, xenongroups] = FindXenonThresholdKmeans(Xe_lung);
            [Xe_rgb_kmeans , UnventilatedMap_Xe_kmeans ,  LowVentMap_Xe_kmeans , MiddleVentMap_Xe_kmeans , HighVentMap_Xe_kmeans] = PlotRGB_Xenon_kmeans(patients(i,:),PlotRGBImageBool,SaveRGBImageBool,xenon_vent_map);
        end
        %% Plot MIP Image on Figure 1 if Selected
        if PlotMIPImageBool
            PlotMIPImageXe(patients(i,:), SaveMIPImageBool, Xe_lung, low_vent_Xe(i), high_vent_Xe(i))
        end
        %% Create and Plot RGB Maps on Figure 2 if selected
        if XenonThresholdNormalBool
            [Xe_rgb , UnventilatedMap_Xe ,  LowVentMap_Xe , MiddleVentMap_Xe , HighVentMap_Xe] = PlotRGB_Xe129(patients(i,:),PlotRGBImageBool,SaveRGBImageBool,Xe_lung, background_Xe(i), low_vent_Xe(i), mid_vent_Xe(i), high_vent_Xe(i));
        end
        if SaveUnventilatedMap
            VarDirectory = strcat('./outputs/UnventilatedMap/');  mkdir(VarDirectory);
            VarName = strcat('UnventilatedMap_Xe_Patient',string(patients(i,:)));
            FileName = char(strcat(VarDirectory,VarName,'.mat'));
            save(FileName,'UnventilatedMap_Xe')
        end
        
        %% Return home
        cd(home)
        
        %% Compute Ventilated Volumes By Type
        if XenonThresholdNormalBool
            AnatomicVolumes_Xe(i)         = sum(moving(:))            *0.3125*0.3125*1.5;
            UnventilatedVolumes_Xe(i)     = sum(UnventilatedMap_Xe(:))*0.3125*0.3125*1.5;
            LowVentilatedVolumes_Xe(i)    = sum(LowVentMap_Xe(:))     *0.3125*0.3125*1.5;
            MiddleVentilatedVolumes_Xe(i) = sum(MiddleVentMap_Xe(:))  *0.3125*0.3125*1.5;
            HighVentilatedVolumes_Xe(i)   = sum(HighVentMap_Xe(:))    *0.3125*0.3125*1.5;
            SNR_Xe(i) = max_vent_Xe(i)/background_Xe(i);
        end
        if XenonKmeansBool
            AnatomicVolumes_Xe_kmeans(i)         = sum(moving(:))            *0.3125*0.3125*1.5;
            UnventilatedVolumes_Xe_kmeans(i)     = sum(UnventilatedMap_Xe_kmeans(:))*0.3125*0.3125*1.5;
            LowVentilatedVolumes_Xe_kmeans(i)    = sum(LowVentMap_Xe_kmeans(:))     *0.3125*0.3125*1.5;
            MiddleVentilatedVolumes_Xe_kmeans(i) = sum(MiddleVentMap_Xe_kmeans(:))  *0.3125*0.3125*1.5;
            HighVentilatedVolumes_Xe_kmeans(i)   = sum(HighVentMap_Xe_kmeans(:))    *0.3125*0.3125*1.5;
        end
        if XenonThresholdNormalBool
            VDP_Xe = (100*UnventilatedVolumes_Xe./AnatomicVolumes_Xe)'
            meanVDP_Xe = mean(VDP_Xe)
            stdVDP_Xe  = std(VDP_Xe)
            LVP_Xe = (100*(UnventilatedVolumes_Xe+LowVentilatedVolumes_Xe)./AnatomicVolumes_Xe)'
            meanLVP = mean(LVP_Xe)
            stdLVP  = std(LVP_Xe)
            max_vent_Xe';
            meanMaxVent_Xe = mean(max_vent_Xe)
        end
        if XenonKmeansBool
            VDP_Xe_kmeans = (100*UnventilatedVolumes_Xe_kmeans./AnatomicVolumes_Xe_kmeans)'
            meanVDP_Xe_kmeans = mean(VDP_Xe_kmeans)
            stdVDP_Xe_kmeans  = std(VDP_Xe_kmeans)
            LVP_Xe_kmeans = (100*(UnventilatedVolumes_Xe_kmeans+LowVentilatedVolumes_Xe_kmeans)./AnatomicVolumes_Xe_kmeans)'
            meanLVP_kmeans = mean(LVP_Xe_kmeans)
            stdLVP_kmeans  = std(LVP_Xe_kmeans)
        end
        if PlotVDPBool
            PlotVDP( patients(i,:),fixed,UnventilatedMap_Xe, 0);
        end
        if PlotMIPBool
            PlotMIP( patients(i,:), FUNC, ANAT, 0)
        end
    end
   
end
if SaveVDPandThresh
    VarDirectory = strcat('./outputs/VentMaps/');  mkdir(VarDirectory);
    VarName = strcat('VDPinfo');
    FileName = char(strcat(VarDirectory,VarName,'.mat'));
    save(FileName,'VDP_Xe','LVP_Xe')
end
VarDirectory = strcat('./outputs/');  mkdir(VarDirectory);
VarName = strcat('SNR');
FileName = char(strcat(VarDirectory,VarName,'.mat'));
save(FileName,'SNR_Xe')
