function [ df_map , tau1_map , r2_Washin_map, d0_map] = WashinFitProcess( image , roi , scantimes , first_pfp, last_pfp )
%UNTITLED3 Summary of this function goes here
%   Detailed explanation goes here

%% Resize roi
image=imresize(image,[size(roi,1),size(roi,2)]);
image_wi = image(:,:,:,1:last_pfp);
vals_washin = scantimes(1:last_pfp);

%% Start Timer
fprintf('\nBeginning F19 Washin Fit.'); timeStart = tic;

%% Define Derived Variables
nrows     = size(image_wi, 1);         % number of rows in image
ncols     = size(image_wi, 2);         % number of cols in image
nslices   = size(image_wi, 3);         % number of slices in image
nscans    = size(image_wi, 4);         % number of scans in time
nel       = nrows*ncols*nslices;    % number of elements in image

%% Mask from ROI
 Mask = roi > 0;
%% Fit WashIn
[ df_map , tau1_map , r2_Washin_map , d0_map ] =   fitwashin( image_wi, Mask, vals_washin, first_pfp);

%% Stop Timer
fprintf('\nFinished F19 Processing.\nTotal Time %0.1f Minutes.',toc(timeStart)/60)

end

function [ df_map , tau1_map , r2_Washin_map , d0_map ] = fitwashin( image, mask, time_vals, first_pfp)
%Computes washin and washout fits and outputs parameter maps
%Splits the time data into a washout and washin section

%% Start Timer
fprintf('\nComputing fit maps for images:'); tfitStart = tic;

%% Initialize Parameter Maps
df_map          = zeros(size(mask));       %Map of df
tau1_map        = zeros(size(mask));       %Map of tau1
r2_Washin_map   = zeros(size(mask));       %Map of r-squared washin
d0_map          = zeros(size(mask));       %Map of d0

%% Define Fit Types                 
WashinFit = fittype('(x<t0)*d0 + (x>=t0)*[d0 + (df-d0)*[1-exp(-(x-t0)/tau1)]]', ... 
    'dependent', {'y'}, 'independent', {'x'}, ...
    'coefficients', {'d0', 'df', 'tau1', 't0'});

%% Set Up Upper and Lower Limits for Fitting
% set up limits for washin
%last_tau1 = f4.tau1;
firstImage = image(:,:,:,1);
lastImage = image(:,:,:,end);
d0_mean = mean(firstImage(:));
d0_std = std(firstImage(:));
d0_lowerlimit = d0_mean-d0_std;
d0_upperlimit = d0_mean+d0_std;

dF_lowerlimit = d0_upperlimit;
dF_upperlimit = max(image(:))*1.5;
est_dF_mean = sum(lastImage(:))/sum(mask(:));
Washin_lower_limits = [ d0_lowerlimit, dF_lowerlimit, 0.5, time_vals(first_pfp)];  % dF and tau 1
Washin_upper_limits = [ d0_upperlimit, dF_upperlimit, 750, time_vals(first_pfp+2)];
Washin_start        = [ d0_mean,       est_dF_mean,   20 , time_vals(first_pfp+1)]
                 
                 
%% Loop through all voxels and compute fit
size(mask)
size(image)
for c = 1:size(mask,3)
    fprintf('\n   Computing Slice %i of %i...',c,size(mask,3)); tStart = tic; % starts timer
    for a = 1:size(mask,1)
        for b = 1:size(mask,2)
            if mask(a,b,c)>0
                %% F19 Washin Fit
                [fit_Washin , gof_Washin] = fit(time_vals(:), squeeze(image(a,b,c,:)), WashinFit, ...
                    'Lower',        Washin_lower_limits, ...
                    'Upper',        Washin_upper_limits, ...
                    'Startpoint',   Washin_start);

                %last_tau1 = fit_Washin.tau1;
                % forces fit to go through 2nd PFP point

                %% Assign Parameter Map Values
                df_map(a,b,c)        = fit_Washin.df;
                tau1_map(a,b,c)      = fit_Washin.tau1;
                r2_Washin_map(a,b,c) = gof_Washin.rsquare;
                d0_map(a,b,c)        = fit_Washin.d0;
            end
        end 
    end
    fprintf('done (%0.1f Seconds)',toc(tStart)) % print timing
end

%% Stop Timer
fprintf('\nFit Maps Completed (total time: %0.1f Minutes)',toc(tfitStart)/60)

end
