%% Initialize
clear;clc;
home = pwd;


%% Choose Parameters for Running
% Choose patients
patients = ['005'; '006'; '007'; '008'; '009'; '010'; '011'; '012'; '013'; '015'];
PlotMIPImageBool = 0;
SaveMIPImageBool = 0;
% RGB Image
PlotRGBImageBool = 0;
SaveRGBImageBool = 0;
% Save CSV ventilation data to file
WriteCSVVentilationDataBool = 0;
% Save tau1 data to file
WriteTau1DataBool = 0;
LoadTau1DataBool = 0;
% B1correction
B1correctBool = 0;
XeCalcHResBool = 0;
XeCalcLResBool = 1;
SaveBiasFieldBool = 0;
MIPThresholdNormalBool = 1;
XenonThresholdNormalBool = 1;
XenonKmeansBool = 0;
MIPKmeansBool = 0;
KmeansTauBool = 0;
SaveKmeansVentMap = 0;
SaveVDPandThresh = 1;
SaveCorrLung = 0;
addpath('./functions')
addpath('./MICO_v0/MICO_v0/MICO_3D')
PlotVDPBool = 0;
PlotMIPBool = 0;

%% Loop through selected subjects
for i = 1:size(patients,1)
    if XeCalcLResBool
        %% Load and Format Low Res Imaging Data
        % load 129Xe ventilaion
        cd('./data')
        filename = strcat('2569-',patients(i,:),'_XE_lowres.mat');
        load(filename);
        fixed = double(FUNC);
        cd(home)
        
        % load anatomical 1h mri mask
        cd('./data')
        filename = strcat('2569-',patients(i,:),'_ROI_lowres.mat');
        load(filename);
        
        moving = double(imresize3(ANATOM_ROI,[size(FUNC,1),size(FUNC,2),size(FUNC,3)]));
        
        % back to home directory and add functions path
        cd(home)
        addpath('./functions')
        
        %% Bias Correction
        if B1correctBool
            [fixed,b] = MICO_3D_lung(fixed,moving);
            if SaveBiasFieldBool
                VarDirectory = strcat('./outputs/BiasField_Xe129/');  mkdir(VarDirectory);
                VarName = strcat('BiasField_Patient_',string(patients(i,:)));
                FileName = char(strcat(VarDirectory,VarName,'_lowres.mat'));
                save(FileName,'b','moving','fixed')
            end
        end
        Xe_lung = fixed.*moving;
        if SaveCorrLung
            VarDirectory = strcat('./outputs/MIP_Xe129/');  mkdir(VarDirectory);
            VarName = strcat('Xe_lung_',string(patients(i,:)));
            FileName = char(strcat(VarDirectory,VarName,'_lowres.mat'));
            save(FileName,'Xe_lung')
        end
        %% Compute Values for lowvent, midvent, highvent
        if XenonThresholdNormalBool
            [background_Xe_lowres(i),low_vent_Xe_lowres(i), mid_vent_Xe_lowres(i), high_vent_Xe_lowres(i), max_vent_Xe_lowres(i)] = FindMIPThresholdValuesLR(fixed , Xe_lung);
        end
        
        background = reshape(fixed(:,5:15,1),1,[]);
        bgd_mean(i,1) = mean(background);
        bgd_std(i,1)  =  std(background);
        %% Compute maps for lowvent, midvent, highvent using kmeans
        if XenonKmeansBool
            [xenon_vent_map, xenongroups] = FindXenonThresholdKmeans(Xe_lung);
            if SaveKmeansVentMap
                VarDirectory = strcat('./outputs/VentMaps/');  mkdir(VarDirectory);
                VarName = strcat('VentMapKmeans_Patient_',string(patients(i,:)));
                FileName = char(strcat(VarDirectory,VarName,'_lowres.mat'));
                save(FileName,'xenon_vent_map')
            end
            [Xe_rgb_kmeans , UnventilatedMap_Xe_kmeans ,  LowVentMap_Xe_kmeans , MiddleVentMap_Xe_kmeans , HighVentMap_Xe_kmeans] = PlotRGB_Xenon_kmeans_lowres(patients(i,:),PlotRGBImageBool,SaveRGBImageBool,xenon_vent_map);
        end
        %% Plot MIP Image on Figure 1 if Selected
        if PlotMIPImageBool
            PlotMIPImageXe(patients(i,:), SaveMIPImageBool, Xe_lung, low_vent_Xe(i), high_vent_Xe(i))
        end
        %% Create and Plot RGB Maps on Figure 2 if selected
        if XenonThresholdNormalBool
            [Xe_rgb , UnventilatedMap_Xe ,  LowVentMap_Xe , MiddleVentMap_Xe , HighVentMap_Xe] = PlotRGB_Xe129_lowres(patients(i,:),PlotRGBImageBool,SaveRGBImageBool,Xe_lung, background_Xe_lowres(i), low_vent_Xe_lowres(i), mid_vent_Xe_lowres(i), high_vent_Xe_lowres(i));
        end
        %% Return home
        cd(home)
        %% Compute Ventilated Volumes By Type
        if XenonThresholdNormalBool
            AnatomicVolumes_Xe_lowres(i)         = sum(moving(:))            *0.3125*0.3125*1.5;
            UnventilatedVolumes_Xe_lowres(i)     = sum(UnventilatedMap_Xe(:))*0.3125*0.3125*1.5;
            LowVentilatedVolumes_Xe_lowres(i)    = sum(LowVentMap_Xe(:))     *0.3125*0.3125*1.5;
            MiddleVentilatedVolumes_Xe_lowres(i) = sum(MiddleVentMap_Xe(:))  *0.3125*0.3125*1.5;
            HighVentilatedVolumes_Xe_lowres(i)   = sum(HighVentMap_Xe(:))    *0.3125*0.3125*1.5;
            SNR_lowres(i) = max_vent_Xe_lowres(i)/background_Xe_lowres(i);
        end
        if XenonKmeansBool
            AnatomicVolumes_Xe_kmeans_lowres(i)         = sum(moving(:))            *0.3125*0.3125*1.5;
            UnventilatedVolumes_Xe_kmeans_lowres(i)     = sum(UnventilatedMap_Xe_kmeans(:))*0.3125*0.3125*1.5;
            LowVentilatedVolumes_Xe_kmeans_lowres(i)    = sum(LowVentMap_Xe_kmeans(:))     *0.3125*0.3125*1.5;
            MiddleVentilatedVolumes_Xe_kmeans_lowres(i) = sum(MiddleVentMap_Xe_kmeans(:))  *0.3125*0.3125*1.5;
            HighVentilatedVolumes_Xe_kmeans_lowres(i)   = sum(HighVentMap_Xe_kmeans(:))    *0.3125*0.3125*1.5;
        end
        if XenonThresholdNormalBool
            VDP_Xe_lowres = (100*UnventilatedVolumes_Xe_lowres./AnatomicVolumes_Xe_lowres)'
            meanVDP_Xe_lowres = mean(VDP_Xe_lowres)
            stdVDP_Xe_lowres  = std(VDP_Xe_lowres)
            LVP_Xe_lowres = (100*(UnventilatedVolumes_Xe_lowres+LowVentilatedVolumes_Xe_lowres)./AnatomicVolumes_Xe_lowres)'
            meanLVP_lowres = mean(LVP_Xe_lowres)
            stdLVP_lowres  = std(LVP_Xe_lowres)
        end
        if XenonKmeansBool
            VDP_Xe_kmeans_lowres = (100*UnventilatedVolumes_Xe_kmeans_lowres./AnatomicVolumes_Xe_kmeans_lowres)'
            meanVDP_Xe_kmeans_lowres = mean(VDP_Xe_kmeans_lowres)
            stdVDP_Xe_kmeans_lowres  = std(VDP_Xe_kmeans_lowres)
            LVP_Xe_kmeans_lowres = (100*(UnventilatedVolumes_Xe_kmeans_lowres+LowVentilatedVolumes_Xe_kmeans_lowres)./AnatomicVolumes_Xe_kmeans_lowres)'
            meanLVP_kmeans_lowres = mean(LVP_Xe_kmeans_lowres)
            stdLVP_kmeans_lowres  = std(LVP_Xe_kmeans_lowres)
        end
        if PlotVDPBool
            PlotVDP( patients(i,:),fixed,UnventilatedMap_Xe, 0);
        end
        if PlotMIPBool
            PlotMIP( patients(i,:), FUNC, ANAT, 1)
        end
    end
end
if SaveVDPandThresh
    VarDirectory = strcat('./outputs/VentMaps/');  mkdir(VarDirectory);
    VarName = strcat('VDPinfo');
    FileName = char(strcat(VarDirectory,VarName,'.mat'));
    save(FileName,'VDP_Xe_lowres','LVP_Xe_lowres')
end
VarDirectory = strcat('./outputs/');  mkdir(VarDirectory);
VarName = strcat('SNR');
FileName = char(strcat(VarDirectory,VarName,'.mat'));
save(FileName,'SNR_lowres')
